@@ -4,7 +4,7 @@
     startup.txt <- paste("\n   /// adegenet ", pkg.version, " is loaded ////////////",
                          "\n\n   > overview: '?adegenet'",
                          "\n   > tutorials/doc/questions: 'adegenetWeb()' ",
-                         "\n   > bug reports/feature resquests: adegenetIssues()\n\n", sep="")
+                         "\n   > bug reports/feature requests: adegenetIssues()\n\n", sep="")
 
     packageStartupMessage(startup.txt)
 }
